@@ -290,6 +290,7 @@ pub fn functions() -> HashSet<&'static str>
         "remove",
         "extend",
         "link",
+        "rationalize",
     ]
     .iter()
     .cloned()
@@ -518,6 +519,21 @@ pub fn functions_with_args() -> HashSet<&'static str>
         "element(A,b)",
         "remove(vec,num/vec)",
         "extend(vec,num/vec)",
+        "rand",
+        "epoch",
+        "inf",
+        "true",
+        "false",
+        "nan",
+        "and",
+        "or",
+        "not",
+        "xor",
+        "nand",
+        "implies",
+        "nor",
+        "converse",
+        "rationalize(x)",
     ]
     .iter()
     .cloned()
