@@ -21,8 +21,8 @@ use crate::{
     load_vars::{add_var, get_cli_vars, get_file_vars, get_vars, set_commands_or_vars},
     math::do_math,
     misc::{
-        clear, clearln, convert, get_terminal_dimensions, handle_err, insert_last, prompt,
-        read_single_char, to_output, write,
+        clear, clearln, convert, end_word, get_terminal_dimensions, handle_err, insert_last,
+        prompt, read_single_char, to_output, write,
     },
     options::{arg_opts, commands, equal_to, file_opts, silent_commands},
     parse::input_var,
@@ -36,6 +36,7 @@ use crossterm::{
 };
 use rug::Complex;
 use std::{
+    cmp::Ordering,
     env::args,
     fs::{File, OpenOptions},
     io::{stdin, stdout, BufRead, BufReader, IsTerminal, Stdout, Write},
@@ -80,20 +81,11 @@ pub struct Colors
     sci: String,
     units: String,
     brackets: Vec<String>,
-    re1col: String,
-    re2col: String,
-    re3col: String,
-    re4col: String,
-    re5col: String,
-    re6col: String,
-    im1col: String,
-    im2col: String,
-    im3col: String,
-    im4col: String,
-    im5col: String,
-    im6col: String,
+    recol: Vec<String>,
+    imcol: Vec<String>,
     label: (String, String, String),
     graphtofile: String,
+    default_units: Vec<(String, Number)>,
 }
 impl Default for Colors
 {
@@ -113,20 +105,25 @@ impl Default for Colors
                 "\x1b[95m".to_string(),
                 "\x1b[96m".to_string(),
             ],
-            re1col: "#ff5555".to_string(),
-            re2col: "#5555ff".to_string(),
-            re3col: "#ff55ff".to_string(),
-            re4col: "#55ff55".to_string(),
-            re5col: "#55ffff".to_string(),
-            re6col: "#ffff55".to_string(),
-            im1col: "#aa0000".to_string(),
-            im2col: "#0000aa".to_string(),
-            im3col: "#aa00aa".to_string(),
-            im4col: "#00aa00".to_string(),
-            im5col: "#00aaaa".to_string(),
-            im6col: "#aaaa00".to_string(),
+            recol: vec![
+                "#ff5555".to_string(),
+                "#5555ff".to_string(),
+                "#ff55ff".to_string(),
+                "#55ff55".to_string(),
+                "#55ffff".to_string(),
+                "#ffff55".to_string(),
+            ],
+            imcol: vec![
+                "#aa0000".to_string(),
+                "#0000aa".to_string(),
+                "#aa00aa".to_string(),
+                "#00aa00".to_string(),
+                "#00aaaa".to_string(),
+                "#aaaa00".to_string(),
+            ],
             label: ("x".to_string(), "y".to_string(), "z".to_string()),
             graphtofile: String::new(),
+            default_units: Vec::new(),
         }
     }
 }
@@ -170,6 +167,8 @@ pub enum Notation
 pub enum GraphType
 {
     Normal,
+    Domain,
+    DomainAlt,
     Flat,
     Depth,
     None,
@@ -200,7 +199,7 @@ pub struct Options
     vzr: (f64, f64),
     samples_2d: usize,
     samples_3d: (usize, usize),
-    point_style: char,
+    point_style: isize,
     lines: Auto,
     multi: bool,
     tabbed: bool,
@@ -216,6 +215,8 @@ pub struct Options
     si_units: bool,
     window_size: (usize, usize),
     keep_zeros: bool,
+    progress: bool,
+    keep_data_file: bool,
 }
 impl Default for Options
 {
@@ -249,7 +250,7 @@ impl Default for Options
             vzr: (0.0, 0.0),
             samples_2d: 8192,
             samples_3d: (256, 256),
-            point_style: '.',
+            point_style: 0,
             lines: Auto::Auto,
             multi: true,
             tabbed: false,
@@ -265,6 +266,8 @@ impl Default for Options
             si_units: false,
             window_size: (0, 0),
             keep_zeros: false,
+            progress: false,
+            keep_data_file: false,
         }
     }
 }
@@ -276,26 +279,25 @@ fn main()
     let mut default = false;
     let dir = dirs::config_dir().unwrap().to_str().unwrap().to_owned() + "/kalc";
     std::fs::create_dir_all(dir.clone()).unwrap();
+    let mut check = Vec::new();
     {
         let file_path = dir.clone() + "/kalc.config";
-        if let Err(s) = file_opts(&mut options, &mut colors, &file_path)
+        if let Ok(s) = file_opts(
+            &mut options,
+            &mut colors,
+            &file_path,
+            &Vec::new(),
+            Vec::new(),
+            true,
+        )
         {
-            println!("{}", s);
-            std::process::exit(1);
+            check = s;
         }
-        match arg_opts(&mut options, &mut colors, &mut args)
+        if let Ok(s) = arg_opts(&mut options, &mut colors, &mut args, &Vec::new(), true)
         {
-            Ok(s) =>
+            if s
             {
-                if s
-                {
-                    default = true
-                }
-            }
-            Err(s) =>
-            {
-                println!("{}", s);
-                std::process::exit(1);
+                default = true
             }
         }
     }
@@ -361,9 +363,20 @@ fn main()
     };
     let mut err = false;
     let base = options.base;
+    let mut argsj = args.join(" ");
+    if !check.is_empty()
+    {
+        argsj += " ";
+        let file_path = dir.clone() + "/kalc.config";
+        argsj += &BufReader::new(File::open(file_path).unwrap())
+            .lines()
+            .map(|a| a.unwrap())
+            .collect::<Vec<String>>()
+            .join(" ");
+    }
     if !options.interactive && options.allow_vars && !options.stay_interactive
     {
-        get_cli_vars(options, args.join(" "), &mut vars)
+        get_cli_vars(options, argsj.clone(), &mut vars)
     }
     {
         if options.allow_vars && !default
@@ -386,7 +399,6 @@ fn main()
                     })
                     .collect::<Vec<String>>();
                 let mut split;
-                let args = args.join(" ");
                 let mut blacklist = if options.interactive || options.stay_interactive
                 {
                     Vec::new()
@@ -414,7 +426,7 @@ fn main()
                         || (!blacklist.contains(&l) && {
                             let mut b = false;
                             let mut word = String::new();
-                            for c in args.chars()
+                            for c in argsj.chars()
                             {
                                 if c.is_alphanumeric() || matches!(c, '\'' | '`' | '_')
                                 {
@@ -501,6 +513,22 @@ fn main()
             }
         }
     }
+    {
+        let file_path = dir.clone() + "/kalc.config";
+        if !check.is_empty()
+        {
+            if let Err(s) = file_opts(&mut options, &mut colors, &file_path, &vars, check, false)
+            {
+                println!("{}", s);
+                std::process::exit(1);
+            }
+        }
+        if let Err(s) = arg_opts(&mut options, &mut colors, &mut args, &vars, false)
+        {
+            println!("{}", s);
+            std::process::exit(1);
+        }
+    }
     if options.interactive && err
     {
         print!(
@@ -706,6 +734,9 @@ fn main()
             let mut end = 0;
             let mut slow = false;
             let mut firstslow = false;
+            let mut xxbool = false;
+            let mut xxstart = false;
+            let mut xxpos = 0;
             loop
             {
                 let c = read_single_char();
@@ -771,26 +802,7 @@ fn main()
                     '\x03' =>
                     {
                         //ctrl+backspace
-                        if placement != 0
-                            && matches!(
-                                input[placement - 1],
-                                '(' | '{'
-                                    | '['
-                                    | ')'
-                                    | '}'
-                                    | ']'
-                                    | '+'
-                                    | '-'
-                                    | '*'
-                                    | '/'
-                                    | '^'
-                                    | '<'
-                                    | '='
-                                    | '>'
-                                    | '|'
-                                    | '&'
-                                    | '!'
-                            )
+                        if placement != 0 && end_word(input[placement - 1])
                         {
                             placement -= 1;
                             input.remove(placement);
@@ -805,25 +817,7 @@ fn main()
                                     placement -= i + 1;
                                     break;
                                 }
-                                if matches!(
-                                    c,
-                                    '(' | '{'
-                                        | '['
-                                        | ')'
-                                        | '}'
-                                        | ']'
-                                        | '+'
-                                        | '-'
-                                        | '*'
-                                        | '/'
-                                        | '^'
-                                        | '<'
-                                        | '='
-                                        | '>'
-                                        | '|'
-                                        | '&'
-                                        | '!'
-                                )
+                                if end_word(*c)
                                 {
                                     input.drain(placement - i..placement);
                                     placement -= i;
@@ -1135,7 +1129,10 @@ fn main()
                     {
                         //ctrl+k
                         cut = input.drain(placement..).collect();
-                        end = input.len();
+                        if end >= input.len()
+                        {
+                            end = input.len();
+                        }
                         if options.real_time_output && !slow
                         {
                             execute!(stdout, DisableBlinking).unwrap();
@@ -1228,10 +1225,9 @@ fn main()
                     '\x16' =>
                     {
                         //ctrl+t
-                        if placement + 1 < input.len()
+                        if placement < input.len() && placement != 0
                         {
-                            let char = input.remove(placement);
-                            input.insert(placement + 1, char);
+                            input.swap(placement - 1, placement);
                             if options.real_time_output && !slow
                             {
                                 execute!(stdout, DisableBlinking).unwrap();
@@ -1277,6 +1273,7 @@ fn main()
                     '\x15' =>
                     {
                         //ctrl+l
+                        print!("\x1b[H\x1b[J");
                         if options.real_time_output && !slow
                         {
                             execute!(stdout, DisableBlinking).unwrap();
@@ -1891,6 +1888,352 @@ fn main()
                             }
                         }
                     }
+                    '\x0E' =>
+                    {
+                        //ctrl+xx
+                        if xxbool
+                        {
+                            (placement, xxpos) = (xxpos, placement);
+                            match placement.cmp(&xxpos)
+                            {
+                                Ordering::Greater =>
+                                {
+                                    if placement >= input.len()
+                                    {
+                                        placement = input.len()
+                                    }
+                                    if placement >= end
+                                    {
+                                        start = placement.saturating_sub(
+                                            get_terminal_dimensions().0
+                                                - if options.prompt { 3 } else { 1 },
+                                        );
+                                        end = placement;
+                                        clearln(&input, &vars, start, end, options, &colors)
+                                    }
+                                    else if placement == xxpos
+                                    {
+                                        placement = input.len();
+                                        print!("\x1b[{}C", input.len() - xxpos);
+                                    }
+                                    else
+                                    {
+                                        print!("\x1b[{}C", placement - xxpos);
+                                    }
+                                }
+                                Ordering::Less =>
+                                {
+                                    if placement <= start
+                                    {
+                                        end = placement
+                                            + (get_terminal_dimensions().0
+                                                - if options.prompt { 3 } else { 1 });
+                                        if end > input.len()
+                                        {
+                                            end = input.len()
+                                        }
+                                        start = placement;
+                                        clearln(&input, &vars, start, end, options, &colors);
+                                        if end - placement != 0
+                                        {
+                                            print!("\x1b[{}D", end - placement)
+                                        }
+                                    }
+                                    else if placement == xxpos
+                                    {
+                                        placement = 0;
+                                        print!("\x1b[{}D", xxpos);
+                                    }
+                                    else
+                                    {
+                                        print!("\x1b[{}D", xxpos - placement);
+                                    }
+                                }
+                                _ =>
+                                {}
+                            }
+                            xxbool = false
+                        }
+                        else
+                        {
+                            xxbool = true;
+                            continue;
+                        }
+                        if xxstart
+                        {
+                            xxpos = 0;
+                        }
+                        xxstart = !xxstart;
+                    }
+                    '\x0D' =>
+                    {
+                        //ctrl+w
+                        if placement != 0 && end_word(input[placement - 1])
+                        {
+                            placement -= 1;
+                            cut = vec![input.remove(placement)];
+                        }
+                        else
+                        {
+                            for (i, c) in input[..placement].iter().rev().enumerate()
+                            {
+                                if c.is_whitespace() || i + 1 == placement
+                                {
+                                    cut = input
+                                        .drain(placement - i - 1..placement)
+                                        .collect::<Vec<char>>();
+                                    placement -= i + 1;
+                                    break;
+                                }
+                                if end_word(*c)
+                                {
+                                    cut = input
+                                        .drain(placement - i..placement)
+                                        .collect::<Vec<char>>();
+                                    placement -= i;
+                                    break;
+                                }
+                            }
+                        }
+                        if end > input.len()
+                        {
+                            end = input.len()
+                        }
+                        if start >= end
+                        {
+                            start = end;
+                        }
+                        if i == lines.len()
+                        {
+                            current.clone_from(&input);
+                        }
+                        else
+                        {
+                            lines[i] = input.clone().iter().collect::<String>();
+                        }
+                        if options.real_time_output && !slow
+                        {
+                            execute!(stdout, DisableBlinking).unwrap();
+                            (frac, graphable, long, varcheck) = print_concurrent(
+                                &input,
+                                &last,
+                                &vars,
+                                options,
+                                colors.clone(),
+                                start,
+                                end,
+                                false,
+                            );
+                            if watch.elapsed().as_millis() > options.slowcheck
+                            {
+                                firstslow = true;
+                                slow = true;
+                            }
+                        }
+                        else if firstslow
+                        {
+                            firstslow = false;
+                            handle_err(
+                                "too slow, will print on enter",
+                                &vars,
+                                &input,
+                                options,
+                                &colors,
+                                start,
+                                end,
+                            )
+                        }
+                        else
+                        {
+                            clearln(&input, &vars, start, end, options, &colors);
+                        }
+                        if options.debug
+                        {
+                            let time = watch.elapsed().as_nanos();
+                            print!(
+                                " {}\x1b[{}D",
+                                time,
+                                time.to_string().len() + 1 + end - placement
+                            );
+                        }
+                        else if end - placement != 0
+                        {
+                            print!("\x1b[{}D", end - placement)
+                        }
+                        if input.is_empty()
+                        {
+                            slow = false;
+                            clear(&input, &vars, start, end, options, &colors);
+                        }
+                    }
+                    '\x0C' =>
+                    {
+                        //alt+d
+                        if placement < input.len() && end_word(input[placement])
+                        {
+                            cut = vec![input.remove(placement)];
+                        }
+                        else
+                        {
+                            let mut pos = 0;
+                            for (i, c) in input[placement..].iter().enumerate()
+                            {
+                                if c.is_whitespace() || placement + i + 1 == input.len()
+                                {
+                                    pos = i + 1;
+                                }
+                                if end_word(*c)
+                                {
+                                    pos = i;
+                                    break;
+                                }
+                            }
+                            cut = input.drain(placement..placement + pos).collect();
+                        }
+                        if end >= input.len()
+                        {
+                            end = input.len();
+                        }
+                        if options.real_time_output && !slow
+                        {
+                            execute!(stdout, DisableBlinking).unwrap();
+                            (frac, graphable, long, varcheck) = print_concurrent(
+                                &input,
+                                &last,
+                                &vars,
+                                options,
+                                colors.clone(),
+                                start,
+                                end,
+                                false,
+                            );
+                            if watch.elapsed().as_millis() > options.slowcheck
+                            {
+                                firstslow = true;
+                                slow = true;
+                            }
+                        }
+                        else if firstslow
+                        {
+                            firstslow = false;
+                            handle_err(
+                                "too slow, will print on enter",
+                                &vars,
+                                &input,
+                                options,
+                                &colors,
+                                start,
+                                end,
+                            )
+                        }
+                        else
+                        {
+                            clearln(&input, &vars, start, end, options, &colors);
+                        }
+                        if end - placement != 0
+                        {
+                            print!("\x1b[{}D", end - placement)
+                        }
+                    }
+                    '\x0F' =>
+                    {
+                        //alt+t
+                        let first;
+                        if placement < input.len() && end_word(input[placement])
+                        {
+                            first = vec![input.remove(placement)];
+                        }
+                        else
+                        {
+                            let mut pos = 0;
+                            for (i, c) in input[placement..].iter().enumerate()
+                            {
+                                if c.is_whitespace() || placement + i + 1 == input.len()
+                                {
+                                    pos = i + 1;
+                                }
+                                if end_word(*c)
+                                {
+                                    pos = i;
+                                    break;
+                                }
+                            }
+                            first = input.drain(placement..placement + pos).collect();
+                        }
+                        let second;
+                        if placement != 0 && end_word(input[placement - 1])
+                        {
+                            second = vec![input.remove(placement)];
+                        }
+                        else
+                        {
+                            let mut pos = 0;
+                            for (i, c) in input[..placement].iter().rev().enumerate()
+                            {
+                                if end_word(*c)
+                                {
+                                    pos = i;
+                                    break;
+                                }
+                                if c.is_whitespace() || i + 1 == placement
+                                {
+                                    pos = i + 1;
+                                    break;
+                                }
+                            }
+                            second = input
+                                .drain(placement - pos..placement)
+                                .collect::<Vec<char>>();
+                        }
+                        placement -= second.len();
+                        input.splice(placement..placement, first.clone());
+                        placement += first.len();
+                        if placement > input.len()
+                        {
+                            placement = input.len() - 1
+                        }
+                        input.splice(placement..placement, second.clone());
+                        if options.real_time_output && !slow
+                        {
+                            execute!(stdout, DisableBlinking).unwrap();
+                            (frac, graphable, long, varcheck) = print_concurrent(
+                                &input,
+                                &last,
+                                &vars,
+                                options,
+                                colors.clone(),
+                                start,
+                                end,
+                                false,
+                            );
+                            if watch.elapsed().as_millis() > options.slowcheck
+                            {
+                                firstslow = true;
+                                slow = true;
+                            }
+                        }
+                        else if firstslow
+                        {
+                            firstslow = false;
+                            handle_err(
+                                "too slow, will print on enter",
+                                &vars,
+                                &input,
+                                options,
+                                &colors,
+                                start,
+                                end,
+                            )
+                        }
+                        else
+                        {
+                            clearln(&input, &vars, start, end, options, &colors);
+                        }
+                        if end - placement != 0
+                        {
+                            print!("\x1b[{}D", end - placement)
+                        }
+                    }
                     '\0' =>
                     {}
                     _ =>
@@ -2054,42 +2397,39 @@ fn main()
                 .split('#')
                 .map(String::from)
                 .collect();
-            if inputs.len() < 7
+            let watch = if options.debug
             {
-                let watch = if options.debug
+                Some(Instant::now())
+            }
+            else
+            {
+                None
+            };
+            if options.graph_cli
+            {
+                if options.interactive
                 {
-                    Some(Instant::now())
+                    terminal::disable_raw_mode().unwrap();
                 }
-                else
+                graph(inputs, vars.clone(), options, watch, colors.clone(), true)
+                    .join()
+                    .unwrap();
+                if options.interactive
                 {
-                    None
-                };
-                if options.graph_cli
-                {
-                    if options.interactive
-                    {
-                        terminal::disable_raw_mode().unwrap();
-                    }
-                    graph(inputs, vars.clone(), options, watch, colors.clone(), true)
-                        .join()
-                        .unwrap();
-                    if options.interactive
-                    {
-                        terminal::enable_raw_mode().unwrap();
-                    }
-                }
-                else
-                {
-                    handles.push(graph(
-                        inputs,
-                        vars.clone(),
-                        options,
-                        watch,
-                        colors.clone(),
-                        false,
-                    ));
+                    terminal::enable_raw_mode().unwrap();
                 }
             }
+            else
+            {
+                handles.push(graph(
+                    inputs,
+                    vars.clone(),
+                    options,
+                    watch,
+                    colors.clone(),
+                    false,
+                ));
+            }
         }
     }
 }
