@@ -1,11 +1,17 @@
 use crate::{
     complex::{
         NumStr,
-        NumStr::{Matrix, Num, Str, Vector},
+        NumStr::{
+            And, Comma, Converse, Conversion, Division, Equal, Exponent, Func, Greater,
+            GreaterEqual, Implies, InternalMultiplication, LeftBracket, LeftCurlyBracket, Lesser,
+            LesserEqual, Matrix, Minus, Modulo, Multiplication, Nand, NearEqual, Nor, Not,
+            NotEqual, Num, Or, Plus, PlusMinus, Range, RightBracket, RightCurlyBracket, Root,
+            ShiftLeft, ShiftRight, Tetration, Vector, Xor,
+        },
     },
     functions::functions,
     math::do_math,
-    print::get_output,
+    print::{custom_units, get_output},
     Colors, Options, Variable,
 };
 use crossterm::event::{read, Event, KeyCode, KeyEvent, KeyEventKind, KeyModifiers};
@@ -151,7 +157,7 @@ pub fn convert(c: &char) -> char
     let valid_chars = [
         '+', '^', '(', ')', '.', '=', ',', '#', '|', '&', '!', '%', '_', '<', '>', ' ', '[', ']',
         '{', '}', '√', '∛', '⁻', 'ⁱ', '`', '±', '∞', ';', ':', '\'', '⌊', '⌈', '⌉', '⌋', '∫', '°',
-        '$', '¢', '≈', '~',
+        '$', '¢', '≈', '~', '¬',
     ];
     match c
     {
@@ -198,6 +204,10 @@ pub fn read_single_char() -> char
                     (KeyCode::Char('d'), KeyModifiers::CONTROL) => '\x06',
                     (KeyCode::Char('p'), KeyModifiers::CONTROL) => '\x05',
                     (KeyCode::Char('n'), KeyModifiers::CONTROL) => '\x04',
+                    (KeyCode::Char('d'), KeyModifiers::ALT) => '\x0C',
+                    (KeyCode::Char('w'), KeyModifiers::CONTROL) => '\x0D',
+                    (KeyCode::Char('x'), KeyModifiers::CONTROL) => '\x0E',
+                    (KeyCode::Char('t'), KeyModifiers::ALT) => '\x0F',
                     (KeyCode::Char(c), KeyModifiers::NONE | KeyModifiers::SHIFT) => convert(&c),
                     (KeyCode::Char(c), KeyModifiers::ALT) => digraph(Some(c)),
                     (KeyCode::Esc, _) => digraph(None),
@@ -206,8 +216,10 @@ pub fn read_single_char() -> char
                     (KeyCode::Char('h'), KeyModifiers::CONTROL) => '\x03',
                     (KeyCode::Backspace, KeyModifiers::NONE) => '\x08',
                     (KeyCode::Delete, KeyModifiers::NONE) => '\x7F',
-                    (KeyCode::Left, KeyModifiers::NONE) => '\x1B',
-                    (KeyCode::Right, KeyModifiers::NONE) => '\x1C',
+                    (KeyCode::Left, KeyModifiers::NONE)
+                    | (KeyCode::Char('b'), KeyModifiers::CONTROL) => '\x1B',
+                    (KeyCode::Right, KeyModifiers::NONE)
+                    | (KeyCode::Char('f'), KeyModifiers::CONTROL) => '\x1C',
                     (KeyCode::Up, KeyModifiers::NONE) => '\x1D',
                     (KeyCode::Down, KeyModifiers::NONE) => '\x1E',
                     (KeyCode::Tab, KeyModifiers::NONE) => '\x1F',
@@ -222,6 +234,30 @@ pub fn read_single_char() -> char
         _ => '\0',
     }
 }
+pub fn end_word(c: char) -> bool
+{
+    matches!(
+        c,
+        '(' | '{'
+            | '['
+            | ')'
+            | '}'
+            | ']'
+            | '+'
+            | '-'
+            | '*'
+            | '/'
+            | '^'
+            | '<'
+            | '='
+            | '>'
+            | '|'
+            | '&'
+            | '!'
+            | '±'
+            | '%'
+    )
+}
 pub fn no_col(input: &str, color: bool) -> Vec<char>
 {
     if color
@@ -534,19 +570,186 @@ pub fn place_funcvarxy(
     {
         if !i.0.contains('(')
         {
-            for j in i.1.iter_mut()
+            let mut sum: Vec<(usize, String)> = Vec::new();
+            let mut bracket = 0;
+            let mut j = 0;
+            while i.1.len() > j
             {
-                match j
+                match &i.1[j]
                 {
-                    Str(s) if matches!(s.as_str(), "x" | "y") => *j = num.clone(),
+                    LeftBracket => bracket += 1,
+                    RightBracket => bracket -= 1,
+                    Comma if !sum.is_empty() && sum[0].0 == bracket =>
+                    {
+                        sum.remove(0);
+                    }
+                    Func(s) =>
+                    {
+                        if matches!(s.as_str(), "x" | "y") && !sum.iter().any(|a| a.1 == *s)
+                        {
+                            i.1[j] = num.clone();
+                        }
+                        else
+                        {
+                            match s.as_str()
+                            {
+                                "sum" | "summation" | "prod" | "product" | "Σ" | "Π" | "vec"
+                                | "mat" | "D" | "integrate" | "arclength" | "∫" | "area"
+                                | "surfacearea" | "sarea" | "solve" | "length" | "slope"
+                                | "lim" | "set" | "limit" | "iter" | "extrema"
+                                    if j + 2 < i.1.len()
+                                        && if let Func(s) = &i.1[j + 2]
+                                        {
+                                            matches!(s.as_str(), "x" | "y")
+                                        }
+                                        else
+                                        {
+                                            false
+                                        } =>
+                                {
+                                    bracket += 1;
+                                    j += 3;
+                                    sum.push((
+                                        bracket,
+                                        if let Func(s) = &i.1[j - 1]
+                                        {
+                                            s.to_string()
+                                        }
+                                        else
+                                        {
+                                            String::new()
+                                        },
+                                    ))
+                                }
+                                "surfacearea" | "sarea"
+                                    if j + 4 < i.1.len()
+                                        && if let Func(s) = &i.1[j + 4]
+                                        {
+                                            matches!(s.as_str(), "x" | "y")
+                                        }
+                                        else
+                                        {
+                                            false
+                                        } =>
+                                {
+                                    bracket += 1;
+                                    j += 5;
+                                    sum.push((
+                                        bracket,
+                                        if let Func(s) = &i.1[j - 1]
+                                        {
+                                            s.to_string()
+                                        }
+                                        else
+                                        {
+                                            String::new()
+                                        },
+                                    ))
+                                }
+                                _ =>
+                                {}
+                            }
+                        }
+                    }
                     _ =>
                     {}
                 }
+                j += 1;
             }
         }
     }
     funcvar
 }
+pub fn place_varxy(mut func: Vec<NumStr>, num: NumStr) -> Vec<NumStr>
+{
+    let mut sum: Vec<(usize, String)> = Vec::new();
+    let mut bracket = 0;
+    let mut i = 0;
+    while func.len() > i
+    {
+        match &func[i]
+        {
+            LeftBracket => bracket += 1,
+            RightBracket => bracket -= 1,
+            Comma if !sum.is_empty() && sum[0].0 == bracket =>
+            {
+                sum.remove(0);
+            }
+            Func(s) =>
+            {
+                if matches!(s.as_str(), "x" | "y") && !sum.iter().any(|a| a.1 == *s)
+                {
+                    func[i] = num.clone();
+                }
+                else
+                {
+                    match s.as_str()
+                    {
+                        "sum" | "summation" | "prod" | "product" | "Σ" | "Π" | "vec" | "mat"
+                        | "D" | "integrate" | "arclength" | "∫" | "area" | "surfacearea"
+                        | "sarea" | "solve" | "length" | "slope" | "lim" | "limit" | "set"
+                        | "iter" | "extrema"
+                            if i + 2 < func.len()
+                                && if let Func(s) = &func[i + 2]
+                                {
+                                    matches!(s.as_str(), "x" | "y")
+                                }
+                                else
+                                {
+                                    false
+                                } =>
+                        {
+                            bracket += 1;
+                            i += 3;
+                            sum.push((
+                                bracket,
+                                if let Func(s) = &func[i - 1]
+                                {
+                                    s.to_string()
+                                }
+                                else
+                                {
+                                    String::new()
+                                },
+                            ))
+                        }
+                        "surfacearea" | "sarea"
+                            if i + 4 < func.len()
+                                && if let Func(s) = &func[i + 4]
+                                {
+                                    matches!(s.as_str(), "x" | "y")
+                                }
+                                else
+                                {
+                                    false
+                                } =>
+                        {
+                            bracket += 1;
+                            i += 5;
+                            sum.push((
+                                bracket,
+                                if let Func(s) = &func[i - 1]
+                                {
+                                    s.to_string()
+                                }
+                                else
+                                {
+                                    String::new()
+                                },
+                            ))
+                        }
+                        _ =>
+                        {}
+                    }
+                }
+            }
+            _ =>
+            {}
+        }
+        i += 1;
+    }
+    func
+}
 pub fn place_funcvar(
     mut funcvar: Vec<(String, Vec<NumStr>)>,
     var: &str,
@@ -559,82 +762,61 @@ pub fn place_funcvar(
         {
             if !i.0.contains('(')
             {
-                for j in i.1.iter_mut()
+                let mut sum = Vec::new();
+                let mut bracket = 0;
+                let mut j = 0;
+                while i.1.len() > j
                 {
-                    match j
+                    match &i.1[j]
                     {
-                        Str(s) if s == var => *j = num.clone(),
+                        LeftBracket => bracket += 1,
+                        RightBracket => bracket -= 1,
+                        Comma if sum.contains(&bracket) =>
+                        {
+                            sum.remove(0);
+                        }
+                        Func(s) =>
+                        {
+                            if s == var && sum.is_empty()
+                            {
+                                i.1[j] = num.clone();
+                            }
+                            else
+                            {
+                                match s.as_str()
+                                {
+                                    "sum" | "summation" | "prod" | "product" | "Σ" | "Π"
+                                    | "vec" | "mat" | "D" | "integrate" | "arclength" | "∫"
+                                    | "area" | "solve" | "length" | "slope" | "lim" | "limit"
+                                    | "set" | "iter" | "extrema" | "surfacearea" | "sarea"
+                                        if j + 2 < i.1.len()
+                                            && i.1[j + 2] == Func(var.to_string()) =>
+                                    {
+                                        j += 3;
+                                        sum.push(bracket)
+                                    }
+                                    "surfacearea" | "sarea"
+                                        if j + 4 < i.1.len()
+                                            && i.1[j + 4] == Func(var.to_string()) =>
+                                    {
+                                        j += 5;
+                                        sum.push(bracket)
+                                    }
+                                    _ =>
+                                    {}
+                                }
+                            }
+                        }
                         _ =>
                         {}
                     }
+                    j += 1;
                 }
             }
         }
     }
     funcvar
 }
-pub fn place_varxy(mut func: Vec<NumStr>, num: NumStr) -> Vec<NumStr>
-{
-    let mut sum = Vec::new();
-    let mut bracket = 0;
-    let mut i = 0;
-    while func.len() > i
-    {
-        if let Str(s) = &func[i]
-        {
-            if matches!(s.as_str(), "x" | "y") && sum.is_empty()
-            {
-                func[i] = num.clone();
-            }
-            else
-            {
-                match s.as_str()
-                {
-                    "(" => bracket += 1,
-                    ")" => bracket -= 1,
-                    "," if sum.contains(&bracket) =>
-                    {
-                        sum.remove(0);
-                    }
-                    "sum" | "summation" | "prod" | "product" | "Σ" | "Π" | "vec" | "mat" | "D"
-                    | "integrate" | "arclength" | "∫" | "area" | "surfacearea" | "sarea"
-                    | "solve" | "length" | "slope" | "lim" | "limit" | "iter" | "extrema"
-                        if i + 2 < func.len()
-                            && if let Str(s) = &func[i + 2]
-                            {
-                                matches!(s.as_str(), "x" | "y")
-                            }
-                            else
-                            {
-                                false
-                            } =>
-                    {
-                        i += 3;
-                        sum.push(bracket)
-                    }
-                    "surfacearea" | "sarea"
-                        if i + 4 < func.len()
-                            && if let Str(s) = &func[i + 4]
-                            {
-                                matches!(s.as_str(), "x" | "y")
-                            }
-                            else
-                            {
-                                false
-                            } =>
-                    {
-                        i += 5;
-                        sum.push(bracket)
-                    }
-                    _ =>
-                    {}
-                }
-            }
-        }
-        i += 1;
-    }
-    func
-}
 pub fn place_var(mut func: Vec<NumStr>, var: &str, num: NumStr) -> Vec<NumStr>
 {
     if !var.is_empty()
@@ -644,41 +826,46 @@ pub fn place_var(mut func: Vec<NumStr>, var: &str, num: NumStr) -> Vec<NumStr>
         let mut i = 0;
         while func.len() > i
         {
-            if let Str(s) = &func[i]
+            match &func[i]
             {
-                if s == var && sum.is_empty()
+                LeftBracket => bracket += 1,
+                RightBracket => bracket -= 1,
+                Comma if sum.contains(&bracket) =>
                 {
-                    func[i] = num.clone();
+                    sum.remove(0);
                 }
-                else
+                Func(s) =>
                 {
-                    match s.as_str()
+                    if s == var && sum.is_empty()
                     {
-                        "(" => bracket += 1,
-                        ")" => bracket -= 1,
-                        "," if sum.contains(&bracket) =>
+                        func[i] = num.clone();
+                    }
+                    else
+                    {
+                        match s.as_str()
                         {
-                            sum.remove(0);
+                            "sum" | "summation" | "prod" | "product" | "Σ" | "Π" | "vec"
+                            | "mat" | "D" | "integrate" | "arclength" | "∫" | "area" | "solve"
+                            | "length" | "slope" | "lim" | "set" | "limit" | "iter" | "extrema"
+                            | "surfacearea" | "sarea"
+                                if i + 2 < func.len() && func[i + 2] == Func(var.to_string()) =>
+                            {
+                                i += 3;
+                                sum.push(bracket)
+                            }
+                            "surfacearea" | "sarea"
+                                if i + 4 < func.len() && func[i + 4] == Func(var.to_string()) =>
+                            {
+                                i += 5;
+                                sum.push(bracket)
+                            }
+                            _ =>
+                            {}
                         }
-                        "sum" | "summation" | "prod" | "product" | "Σ" | "Π" | "vec" | "mat"
-                        | "D" | "integrate" | "arclength" | "∫" | "area" | "solve" | "length"
-                        | "slope" | "lim" | "limit" | "iter" | "extrema" | "surfacearea"
-                        | "sarea"
-                            if i + 2 < func.len() && func[i + 2] == Str(var.to_string()) =>
-                        {
-                            i += 3;
-                            sum.push(bracket)
-                        }
-                        "surfacearea" | "sarea"
-                            if i + 4 < func.len() && func[i + 4] == Str(var.to_string()) =>
-                        {
-                            i += 5;
-                            sum.push(bracket)
-                        }
-                        _ =>
-                        {}
                     }
                 }
+                _ =>
+                {}
             }
             i += 1;
         }
@@ -710,7 +897,7 @@ pub fn parsed_to_string(
     let mut i = 0;
     'main: while i < input.len()
     {
-        if let Str(s) = &input[i]
+        if let Func(s) = &input[i]
         {
             for v in &func_vars
             {
@@ -718,17 +905,17 @@ pub fn parsed_to_string(
                 {
                     if i != 0
                         && i + 1 < input.len()
-                        && input[i - 1].str_is("(")
-                        && input[i + 1].str_is(")")
+                        && input[i - 1] == LeftBracket
+                        && input[i + 1] == RightBracket
                     {
                         input.remove(i);
                         input.splice(i..i, v.1.clone());
                     }
                     else
                     {
-                        input[i] = Str('('.to_string());
+                        input[i] = LeftBracket;
                         input.splice(i + 1..i + 1, v.1.clone());
-                        input.insert(i + v.1.len() + 1, Str(')'.to_string()));
+                        input.insert(i + v.1.len() + 1, RightBracket);
                     }
                     continue 'main;
                 }
@@ -743,6 +930,7 @@ pub fn parsed_to_string(
         {
             Num(n) =>
             {
+                let n = custom_units(n, *options, colors);
                 let n = get_output(*options, colors, &n);
                 format!(
                     "{}{}{}{}",
@@ -765,6 +953,7 @@ pub fn parsed_to_string(
                 let mut num;
                 for i in n
                 {
+                    let i = custom_units(i, *options, colors);
                     num = get_output(*options, colors, &i);
                     str.push_str(&format!(
                         "{}{}{}{},",
@@ -793,6 +982,7 @@ pub fn parsed_to_string(
                     str.push('{');
                     for j in i
                     {
+                        let j = custom_units(j, *options, colors);
                         num = get_output(*options, colors, &j);
                         str.push_str(&format!(
                             "{}{}{}{},",
@@ -814,11 +1004,45 @@ pub fn parsed_to_string(
                 str.pop();
                 format!("{{{}}}", str)
             }
-            Str(n) if n.starts_with('@') && n.contains('(') =>
+            Func(n) if n.starts_with('@') && n.contains('(') =>
             {
                 n.split('(').next().unwrap().replace('@', "")
             }
-            Str(n) => n.replace('@', ""),
+            Func(n) => n.replace('@', ""),
+            LeftBracket => "(".to_string(),
+            RightBracket => ")".to_string(),
+            LeftCurlyBracket => "{".to_string(),
+            RightCurlyBracket => "}".to_string(),
+            Comma => ",".to_string(),
+            Plus => "+".to_string(),
+            Minus => "-".to_string(),
+            PlusMinus => "±".to_string(),
+            Multiplication => "*".to_string(),
+            InternalMultiplication => "×".to_string(),
+            Division => "/".to_string(),
+            Root => "//".to_string(),
+            Tetration => "^^".to_string(),
+            Exponent => "^".to_string(),
+            Equal => "==".to_string(),
+            NotEqual => "!=".to_string(),
+            Greater => ">".to_string(),
+            GreaterEqual => ">=".to_string(),
+            Lesser => "<".to_string(),
+            LesserEqual => "<".to_string(),
+            Modulo => "%".to_string(),
+            Range => "..".to_string(),
+            And => "&&".to_string(),
+            Or => "||".to_string(),
+            ShiftLeft => "<<".to_string(),
+            ShiftRight => ">>".to_string(),
+            Conversion => "->".to_string(),
+            NearEqual => "≈".to_string(),
+            Xor => " Xor ".to_string(),
+            Implies => " Implies ".to_string(),
+            Nand => " Nand ".to_string(),
+            Not => "¬".to_string(),
+            Nor => " Nor ".to_string(),
+            Converse => " Converse ".to_string(),
         })
     }
     to_output(
