@@ -5,9 +5,9 @@ use crate::{
     math::do_math,
     misc::{insert_last, parsed_to_string, to_output},
     parse::input_var,
-    print::get_output,
+    print::{custom_units, get_output},
     AngleType::{Degrees, Gradians, Radians},
-    Auto, Colors, GraphType,
+    Auto, Colors, GraphType, HowGraphing,
     Notation::{LargeEngineering, Normal, Scientific, SmallEngineering},
     Number, Options, Variable,
 };
@@ -15,7 +15,7 @@ use crossterm::{
     execute, terminal,
     terminal::{Clear, ClearType},
 };
-use rug::{Complex, Float};
+use rug::{float::Special::Nan, Complex, Float};
 use std::{
     fs::File,
     io::{BufRead, BufReader, Stdout, Write},
@@ -25,10 +25,15 @@ pub fn arg_opts(
     options: &mut Options,
     colors: &mut Colors,
     args: &mut Vec<String>,
+    vars: &[Variable],
+    soft: bool,
 ) -> Result<bool, &'static str>
 {
     let mut default = false;
-    args.remove(0);
+    if soft
+    {
+        args.remove(0);
+    }
     loop
     {
         if args.is_empty()
@@ -83,16 +88,33 @@ pub fn arg_opts(
                     match set_commands(
                         options,
                         colors,
-                        &mut Vec::new(),
+                        &mut vars.to_vec(),
                         split.next().unwrap(),
                         split.next().unwrap(),
                     )
                     {
-                        Err(s) if !s.is_empty() => return Err(s),
+                        Err(s) if !s.is_empty() =>
+                        {
+                            if soft
+                            {
+                                break;
+                            }
+                            else
+                            {
+                                return Err(s);
+                            }
+                        }
                         Ok(()) =>
                         {
-                            println!("{} failed", arg);
-                            process::exit(1);
+                            if soft
+                            {
+                                break;
+                            }
+                            else
+                            {
+                                println!("{} failed", arg);
+                                process::exit(1);
+                            }
                         }
                         _ =>
                         {}
@@ -100,8 +122,15 @@ pub fn arg_opts(
                 }
                 else if !silent_commands(options, &arg.chars().collect::<Vec<char>>())
                 {
-                    println!("{} failed", args[0]);
-                    process::exit(1);
+                    if soft
+                    {
+                        break;
+                    }
+                    else
+                    {
+                        println!("{} failed", args[0]);
+                        process::exit(1);
+                    }
                 }
                 args.remove(0);
             }
@@ -113,14 +142,21 @@ pub fn file_opts(
     options: &mut Options,
     colors: &mut Colors,
     file_path: &String,
-) -> Result<(), &'static str>
+    vars: &[Variable],
+    check: Vec<usize>,
+    soft: bool,
+) -> Result<Vec<usize>, &'static str>
 {
-    if File::open(file_path).is_ok()
+    let mut err = Vec::new();
+    if let Ok(file) = File::open(file_path)
     {
-        let file = File::open(file_path).unwrap();
         let reader = BufReader::new(file);
-        for line in reader.lines().map(|l| l.unwrap())
+        for (i, line) in reader.lines().map(|l| l.unwrap()).enumerate()
         {
+            if !(check.is_empty() || check.iter().any(|n| n == &i))
+            {
+                continue;
+            }
             if line.starts_with('#') || line.is_empty()
             {
                 continue;
@@ -131,16 +167,33 @@ pub fn file_opts(
                 match set_commands(
                     options,
                     colors,
-                    &mut Vec::new(),
+                    &mut vars.to_vec(),
                     split.next().unwrap(),
                     split.next().unwrap(),
                 )
                 {
-                    Err(s) if !s.is_empty() => return Err(s),
+                    Err(s) if !s.is_empty() =>
+                    {
+                        if soft
+                        {
+                            err.push(i)
+                        }
+                        else
+                        {
+                            return Err(s);
+                        }
+                    }
                     Ok(()) =>
                     {
-                        println!("{} failed", line);
-                        process::exit(1);
+                        if soft
+                        {
+                            err.push(i);
+                        }
+                        else
+                        {
+                            println!("{} failed", line);
+                            process::exit(1);
+                        }
                     }
                     _ =>
                     {}
@@ -148,12 +201,19 @@ pub fn file_opts(
             }
             else if !silent_commands(options, &line.chars().collect::<Vec<char>>())
             {
-                println!("{} failed", line);
-                process::exit(1);
+                if soft
+                {
+                    err.push(i);
+                }
+                else
+                {
+                    println!("{} failed", line);
+                    process::exit(1);
+                }
             }
         }
     }
-    Ok(())
+    Ok(err)
 }
 pub fn set_commands(
     options: &mut Options,
@@ -167,7 +227,7 @@ pub fn set_commands(
     let r = s.as_str();
     match l
     {
-        "color" =>
+        "color" | "colour" =>
         {
             options.color = match r
             {
@@ -216,6 +276,8 @@ pub fn set_commands(
                 "null" | "none" | "false" => GraphType::None,
                 "depth" => GraphType::Depth,
                 "flat" => GraphType::Flat,
+                "domain" | "domain1" | "domain_coloring" => GraphType::Domain,
+                "domain_alt" | "domainalt" | "domain2" => GraphType::DomainAlt,
                 _ => return Err("bad graph type"),
             }
         }
@@ -230,18 +292,42 @@ pub fn set_commands(
                 colors.graphtofile = r.to_string()
             }
         }
-        "re1col" => colors.re1col = r.to_string(),
-        "im1col" => colors.im1col = r.to_string(),
-        "re2col" => colors.re2col = r.to_string(),
-        "im2col" => colors.im2col = r.to_string(),
-        "re3col" => colors.re3col = r.to_string(),
-        "im3col" => colors.im3col = r.to_string(),
-        "re4col" => colors.re4col = r.to_string(),
-        "im4col" => colors.im4col = r.to_string(),
-        "re5col" => colors.re5col = r.to_string(),
-        "im5col" => colors.im5col = r.to_string(),
-        "re6col" => colors.re6col = r.to_string(),
-        "im6col" => colors.im6col = r.to_string(),
+        "recol" =>
+        {
+            colors.recol = r
+                .split(',')
+                .map(|a| {
+                    if a.len() == 6 && a.chars().all(|c| c.is_ascii_hexdigit())
+                    {
+                        Some("#".to_owned() + a)
+                    }
+                    else
+                    {
+                        None
+                    }
+                })
+                .take_while(|a| a.is_some())
+                .map(|a| a.unwrap())
+                .collect::<Vec<String>>()
+        }
+        "imcol" =>
+        {
+            colors.imcol = r
+                .split(',')
+                .map(|a| {
+                    if a.len() == 6 && a.chars().all(|c| c.is_ascii_hexdigit())
+                    {
+                        Some("#".to_owned() + a)
+                    }
+                    else
+                    {
+                        None
+                    }
+                })
+                .take_while(|a| a.is_some())
+                .map(|a| a.unwrap())
+                .collect::<Vec<String>>()
+        }
         "textc" => colors.text = "\x1b[".to_owned() + r,
         "promptc" => colors.prompt = "\x1b[".to_owned() + r,
         "imagc" => colors.imag = "\x1b[".to_owned() + r,
@@ -249,10 +335,59 @@ pub fn set_commands(
         "unitsc" => colors.units = "\x1b[".to_owned() + r,
         "bracketc" =>
         {
-            colors.brackets = r
-                .split(',')
-                .map(|a| "\x1b[".to_owned() + a)
-                .collect::<Vec<String>>()
+            if r == "null"
+            {
+                colors.brackets.clear()
+            }
+            else
+            {
+                colors.brackets = r
+                    .split(',')
+                    .map(|a| "\x1b[".to_owned() + a)
+                    .collect::<Vec<String>>()
+            }
+        }
+        "default_units" =>
+        {
+            if r == "null"
+            {
+                colors.default_units.clear()
+            }
+            else
+            {
+                let n = Number::from(Complex::with_val(options.prec, Nan), None);
+                colors.default_units = r
+                    .split(',')
+                    .map(|a| {
+                        (a.to_string(), {
+                            let i = input_var(
+                                a,
+                                vars,
+                                &mut Vec::new(),
+                                &mut 0,
+                                *options,
+                                false,
+                                0,
+                                Vec::new(),
+                                false,
+                                &mut Vec::new(),
+                                None,
+                            )
+                            .unwrap_or((
+                                Vec::new(),
+                                Vec::new(),
+                                HowGraphing::default(),
+                                false,
+                                None,
+                            ));
+                            do_math(i.0, *options, i.1)
+                                .unwrap_or(Num(n.clone()))
+                                .num()
+                                .unwrap_or(n.clone())
+                        })
+                    })
+                    .collect::<Vec<(String, Number)>>()
+            }
         }
         "label" =>
         {
@@ -280,28 +415,23 @@ pub fn set_commands(
             else
             {
                 let r = r.chars().next().unwrap();
-                if matches!(
-                    r,
-                    '.' | '+'
-                        | 'x'
-                        | '*'
-                        | 's'
-                        | 'S'
-                        | 'o'
-                        | 'O'
-                        | 't'
-                        | 'T'
-                        | 'd'
-                        | 'D'
-                        | 'r'
-                        | 'R'
-                )
+                options.point_style = match r
                 {
-                    options.point_style = r;
-                }
-                else
-                {
-                    return Err("Invalid point type");
+                    '.' => 0,
+                    '+' => 1,
+                    'x' => 2,
+                    '*' => 3,
+                    's' => 4,
+                    'S' => 5,
+                    'o' => 6,
+                    'O' => 7,
+                    't' => 8,
+                    'T' => 9,
+                    'd' => 10,
+                    'D' => 11,
+                    'r' => 12,
+                    'R' => 13,
+                    _ => return Err("invalid point type"),
                 }
             }
         }
@@ -310,13 +440,14 @@ pub fn set_commands(
         | "tabbed" | "comma" | "units" | "scalegraph" | "debug" | "vars" | "onaxis" | "base"
         | "ticks" | "decimal" | "deci" | "decimals" | "graphprec" | "graphprecision" | "prec"
         | "windowsize" | "precision" | "range" | "xr" | "yr" | "zr" | "vrange" | "vxr" | "vyr"
-        | "vzr" | "2d" | "3d" =>
+        | "vzr" | "2d" | "3d" | "progress" =>
         {
             let mut args: Vec<Float> = Vec::new();
             {
                 let mut bracket = 0;
                 let mut last = 0;
-                for (i, c) in o.chars().enumerate()
+                let oc = o.chars().collect::<Vec<char>>();
+                for (i, c) in oc.iter().enumerate()
                 {
                     match c
                     {
@@ -325,7 +456,7 @@ pub fn set_commands(
                         ',' if bracket == 0 =>
                         {
                             let parsed = input_var(
-                                &o[last..i],
+                                &oc[last..i].iter().collect::<String>(),
                                 vars,
                                 &mut Vec::new(),
                                 &mut 0,
@@ -373,11 +504,13 @@ pub fn set_commands(
             }
             match l
             {
+                "keep_data_file" => options.keep_data_file = args[0] != 0.0,
                 "graphcli" => options.graph_cli = args[0] != 0.0,
                 "interactive" => options.stay_interactive = args[0] != 0.0,
                 "prompt" => options.prompt = args[0] != 0.0,
                 "surface" => options.surface = args[0] != 0.0,
                 "rt" => options.real_time_output = args[0] != 0.0,
+                "progress" => options.progress = args[0] != 0.0,
                 "siunits" => options.si_units = args[0] != 0.0,
                 "keepzeros" => options.keep_zeros = args[0] != 0.0,
                 "polar" => options.polar = args[0] != 0.0,
@@ -836,36 +969,68 @@ pub fn set_commands(
                 }
                 "2d" =>
                 {
-                    options.samples_2d = args[0]
-                        .to_integer()
-                        .unwrap_or_default()
-                        .to_usize()
-                        .unwrap_or_default()
+                    if args[0].is_sign_negative()
+                    {
+                        options.samples_2d = (options.xr.1 - options.xr.0) as usize
+                    }
+                    else
+                    {
+                        options.samples_2d = args[0]
+                            .to_integer()
+                            .unwrap_or_default()
+                            .to_usize()
+                            .unwrap_or_default()
+                    }
                 }
                 "3d" =>
                 {
                     if args.len() == 1
                     {
-                        let range = args[0]
-                            .to_integer()
-                            .unwrap_or_default()
-                            .to_usize()
-                            .unwrap_or_default();
-                        options.samples_3d = (range, range)
+                        if args[0].is_sign_negative()
+                        {
+                            options.samples_3d = (
+                                (options.xr.1 - options.xr.0) as usize,
+                                (options.yr.1 - options.yr.0) as usize,
+                            )
+                        }
+                        else
+                        {
+                            let range = args[0]
+                                .to_integer()
+                                .unwrap_or_default()
+                                .to_usize()
+                                .unwrap_or_default();
+                            options.samples_3d = (range, range)
+                        }
                     }
                     else
                     {
-                        let x = args[0]
-                            .to_integer()
-                            .unwrap_or_default()
-                            .to_usize()
-                            .unwrap_or_default();
-                        let y = args[1]
-                            .to_integer()
-                            .unwrap_or_default()
-                            .to_usize()
-                            .unwrap_or_default();
-                        options.samples_3d = (x, y)
+                        if args[0].is_sign_negative()
+                        {
+                            options.samples_3d.0 = (options.xr.1 - options.xr.0) as usize
+                        }
+                        else
+                        {
+                            let range = args[0]
+                                .to_integer()
+                                .unwrap_or_default()
+                                .to_usize()
+                                .unwrap_or_default();
+                            options.samples_3d.0 = range
+                        }
+                        if args[1].is_sign_negative()
+                        {
+                            options.samples_3d.1 = (options.yr.1 - options.yr.0) as usize
+                        }
+                        else
+                        {
+                            let range = args[1]
+                                .to_integer()
+                                .unwrap_or_default()
+                                .to_usize()
+                                .unwrap_or_default();
+                            options.samples_3d.1 = range
+                        }
                     }
                 }
                 _ => return Ok(()),
@@ -883,7 +1048,7 @@ pub fn silent_commands(options: &mut Options, input: &[char]) -> bool
         "interactive" => options.stay_interactive = !options.stay_interactive,
         "scalegraph" => options.scale_graph = !options.scale_graph,
         "debug" => options.debug = !options.debug,
-        "color" =>
+        "color" | "colour" =>
         {
             options.color = match options.color
             {
@@ -895,6 +1060,8 @@ pub fn silent_commands(options: &mut Options, input: &[char]) -> bool
         "onaxis" => options.onaxis = !options.onaxis,
         "surface" => options.surface = !options.surface,
         "rt" => options.real_time_output = !options.real_time_output,
+        "progress" => options.progress = !options.progress,
+        "keep_data_file" => options.keep_data_file = !options.keep_data_file,
         "siunits" => options.si_units = !options.si_units,
         "keepzeros" => options.keep_zeros = !options.keep_zeros,
         "line" | "lines" =>
@@ -942,7 +1109,7 @@ pub fn commands(options: &mut Options, lines: &[String], input: &[char], stdout:
             stdout.flush().unwrap();
             options.scale_graph = !options.scale_graph
         }
-        "color" =>
+        "color" | "colour" =>
         {
             print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
@@ -976,6 +1143,12 @@ pub fn commands(options: &mut Options, lines: &[String], input: &[char], stdout:
             stdout.flush().unwrap();
             options.real_time_output = !options.real_time_output;
         }
+        "progress" =>
+        {
+            print!("\x1b[G\x1b[A\x1b[K");
+            stdout.flush().unwrap();
+            options.progress = !options.progress;
+        }
         "siunits" =>
         {
             print!("\x1b[G\x1b[A\x1b[K");
@@ -1114,7 +1287,7 @@ pub fn list_vars(vars: &[Variable], options: &Options, colors: &Colors) -> Strin
         {
             out += format!(
                 "{}={}\x1b[G\n",
-                v.name.iter().collect::<String>(),
+                to_output(&v.name, vars, options.color == Auto::True, colors,),
                 to_output(
                     &v.unparsed.chars().collect::<Vec<char>>(),
                     vars,
@@ -1130,7 +1303,8 @@ pub fn list_vars(vars: &[Variable], options: &Options, colors: &Colors) -> Strin
             {
                 Num(n) =>
                 {
-                    let n = get_output(*options, colors, n);
+                    let n = custom_units(n.clone(), *options, colors);
+                    let n = get_output(*options, colors, &n);
                     out += format!(
                         "{}={}{}{}{}\x1b[G\n",
                         v.name.iter().collect::<String>(),
@@ -1153,7 +1327,8 @@ pub fn list_vars(vars: &[Variable], options: &Options, colors: &Colors) -> Strin
                     let mut st = String::new();
                     for i in m
                     {
-                        let n = get_output(*options, colors, i);
+                        let i = custom_units(i.clone(), *options, colors);
+                        let n = get_output(*options, colors, &i);
                         st.push_str(&n.0);
                         st.push_str(&n.1);
                         st.push_str(&n.2.unwrap_or_default());
@@ -1178,7 +1353,8 @@ pub fn list_vars(vars: &[Variable], options: &Options, colors: &Colors) -> Strin
                         st.push('{');
                         for g in i
                         {
-                            let n = get_output(*options, colors, g);
+                            let g = custom_units(g.clone(), *options, colors);
+                            let n = get_output(*options, colors, &g);
                             st.push_str(&n.0);
                             st.push_str(&n.1);
                             st.push_str(&n.2.unwrap_or_default());
@@ -1213,7 +1389,7 @@ pub fn equal_to(options: Options, colors: &Colors, vars: &[Variable], l: &str, l
         "colors" =>
         {
             format!(
-            "{}textc={} {}promptc={} {}imagc={} {}scic={} {}unitsc={} \x1b[0mbracketc={} \x1b[0mre1col={} re2col={} re3col={} re4col={} re5col={} re6col={} im1col={} im2col={} im3col={} im4col={} im5col={} im6col={}",
+            "{}textc={} {}promptc={} {}imagc={} {}scic={} {}unitsc={} \x1b[0mbracketc={}\x1b[0m{}{}",
             colors.text,
             &colors.text[2..],
             colors.prompt,
@@ -1225,23 +1401,13 @@ pub fn equal_to(options: Options, colors: &Colors, vars: &[Variable], l: &str, l
             colors.units,
             &colors.units[2..],
             bracketcol(&colors.brackets),
-            formatcol(&colors.re1col),
-            formatcol(&colors.re2col),
-            formatcol(&colors.re3col),
-            formatcol(&colors.re4col),
-            formatcol(&colors.re5col),
-            formatcol(&colors.re6col),
-            formatcol(&colors.im1col),
-            formatcol(&colors.im2col),
-            formatcol(&colors.im3col),
-            formatcol(&colors.im4col),
-            formatcol(&colors.im5col),
-            formatcol(&colors.im6col),
+            colors.recol.iter().enumerate().map(|(i,a)| format!(" re{}col={}",i,formatcol(a))).collect::<Vec<String>>().concat(),
+            colors.imcol.iter().enumerate().map(|(i,a)| format!(" im{}col={}",i,formatcol(a))).collect::<Vec<String>>().concat(),
         )
         }
         "slowcheck" => format!("{}", options.slowcheck),
         "label" => format!("{},{},{}", colors.label.0, colors.label.1, colors.label.2),
-        "color" => (match options.color
+        "color" | "colour" => (match options.color
         {
             Auto::Auto => "auto",
             Auto::False => "false",
@@ -1251,6 +1417,7 @@ pub fn equal_to(options: Options, colors: &Colors, vars: &[Variable], l: &str, l
         "surface" => format!("{}", options.surface),
         "prompt" => format!("{}", options.prompt),
         "rt" => format!("{}", options.real_time_output),
+        "progress" => format!("{}", options.progress),
         "siunits" => format!("{}", options.si_units),
         "keepzeros" => format!("{}", options.keep_zeros),
         "debug" => format!("{}", options.debug),
@@ -1321,6 +1488,8 @@ pub fn equal_to(options: Options, colors: &Colors, vars: &[Variable], l: &str, l
         "graph" => match options.graphtype
         {
             GraphType::Normal => "normal",
+            GraphType::Domain => "domain",
+            GraphType::DomainAlt => "domain_alt",
             GraphType::None => "none",
             GraphType::Depth => "depth",
             GraphType::Flat => "flat",
@@ -1333,19 +1502,27 @@ pub fn equal_to(options: Options, colors: &Colors, vars: &[Variable], l: &str, l
         "scic" => colors.sci.to_string(),
         "unitsc" => colors.units.to_string(),
         "bracketc" => bracketcol(&colors.brackets),
+        "default_units" => colors
+            .default_units
+            .iter()
+            .map(|a| a.0.clone())
+            .collect::<Vec<String>>()
+            .join(","),
         "saveto" => colors.graphtofile.to_string(),
-        "re1col" => formatcol(&colors.re1col),
-        "re2col" => formatcol(&colors.re2col),
-        "re3col" => formatcol(&colors.re3col),
-        "re4col" => formatcol(&colors.re4col),
-        "re5col" => formatcol(&colors.re5col),
-        "re6col" => formatcol(&colors.re6col),
-        "im1col" => formatcol(&colors.im1col),
-        "im2col" => formatcol(&colors.im2col),
-        "im3col" => formatcol(&colors.im3col),
-        "im4col" => formatcol(&colors.im4col),
-        "im5col" => formatcol(&colors.im5col),
-        "im6col" => formatcol(&colors.im6col),
+        "recol" => colors
+            .recol
+            .iter()
+            .enumerate()
+            .map(|(i, a)| format!("re{}col={}", i, formatcol(a)))
+            .collect::<Vec<String>>()
+            .join(" "),
+        "imcol" => colors
+            .imcol
+            .iter()
+            .enumerate()
+            .map(|(i, a)| format!("im{}col={}", i, formatcol(a)))
+            .collect::<Vec<String>>()
+            .join(" "),
         _ =>
         {
             let input = input_var(
